@@ -33,6 +33,14 @@ defaults = {
     # No leading dot, for example: ['avi', 'mkv', 'mp4']
     'valid_extensions': [],
 
+    # Pattern for splitting filenames into basename and extension.
+    # Useful for matching subtitles with language codes, for example
+    # "extension_pattern": "(\.(eng|cze))?(\.[a-zA-Z0-9]+)$" will split "foo.eng.srt"
+    # into "foo" and ".eng.srt".
+    # Note that extensions still pass 'valid_extensions' filter, '.eng.srt' passes
+    # when 'srt' is specified in 'valid_extensions'.
+    'extension_pattern': '(\.[a-zA-Z0-9]+)$',
+
     # When non-empty, filter out filenames that match these expressions. Either simple
     # matches or regexs can be used. The following are near enough equivalent:
     # [{"is_regex": true, "match": ".*sample.*"}, {"is_regex": false, "match": "sample"}]
@@ -48,6 +56,9 @@ defaults = {
     # Convert output filenames to lower case (applied after replacements)
     'lowercase_filename': False,
 
+    # Convert output filenames to 'Title Case' (applied after replacements)
+    'titlecase_filename': False,
+
     # Extra characters to consider invalid in output filenames (which are
     # replaced by the character in replace_invalid_characters_with)
     'custom_filename_character_blacklist': '',
@@ -120,7 +131,11 @@ defaults = {
     # copied).  If True, this will delete the file from the original
     # volume, after the copy has complete.
     'always_move': False,
-    
+
+    # Whenever a file is moved leave a symlink to the new file behind, named
+    # after the original file.
+    'leave_symlink': False,
+
     # Allow user to copy files to specified move location without renaming files.
     'move_files_only': False,
 
@@ -407,27 +422,30 @@ defaults = {
 
 
 
-    # Used to join multiple episode names together
+    # Used to join multiple episode names together (only when episode names are different)
     'multiep_join_name_with': ', ',
 
-    # Format for numbers (python string format), %02d does 2-digit
-    # padding, %d will cause no padding
+    # Format for multi-episode names (only when episode names are the same)
+    # Formats mapping key 'episodename' (used in variables 'filename_with_episode' etc.)
+    'multiep_format': '%(epname)s (%(episodemin)s-%(episodemax)s)',
+
+    # Format for numbers (python string format), %02d does 2-digit padding, %d will cause no padding
     'episode_single': '%02d',
 
-    # String to join multiple number
+    # String to join multiple numbers in mapping key 'episode' (used in variables 'filename_with_episode' etc.)
     'episode_separator': '-',
-    
+
     # Series ID to use instead of searching if the value is set
     #'series_id': None,
-    
+
     # Forced Name to use
     #'forced_name': None,
-    
+
     # replace series names before/after passing to TVDB
     # input replacements are regular expressions for the series as parsed from
     # filenames, for instance adding or removing the year, or expanding abbreviations
     'input_series_replacements': {},
-    
+
     # output replacements are for transforms of the TVDB series names
     # since these are perfectly predictable, they are simple strings
     # not regular expressions
