@@ -6,6 +6,7 @@
 import os
 import sys
 import logging
+import warnings
 
 try:
     import readline
@@ -88,7 +89,7 @@ def doRenameFile(cnamer, newName):
     newName should be string containing new filename.
     """
     try:
-        cnamer.newName(newName, force = Config['overwrite_destination_on_rename'])
+        cnamer.newPath(new_fullpath = newName, force = Config['overwrite_destination_on_rename'], leave_symlink = Config['leave_symlink'])
     except OSError, e:
         warn(e)
 
@@ -111,6 +112,7 @@ def doMoveFile(cnamer, destDir = None, destFilepath = None, getPathPreview = Fal
             new_path = destDir,
             new_fullpath = destFilepath,
             always_move = Config['always_move'],
+            leave_symlink = Config['leave_symlink'],
             getPathPreview = getPathPreview,
             force = Config['overwrite_destination_on_move'])
 
@@ -181,34 +183,34 @@ def processFile(tvdb_instance, episode):
         warn(errormsg)
 
     cnamer = Renamer(episode.fullpath)
-   
+
 
     shouldRename = False
 
     if Config["move_files_only"]:
-        
+
         newName = episode.fullfilename
         shouldRename = True
-        
+
     else:
         newName = episode.generateFilename()
         if newName == episode.fullfilename:
             p("#" * 20)
             p("Existing filename is correct: %s" % episode.fullfilename)
             p("#" * 20)
-    
+
             shouldRename = True
-    
+
         else:
             p("#" * 20)
             p("Old filename: %s" % episode.fullfilename)
-    
+
             if len(Config['output_filename_replacements']) > 0:
                 # Show filename without replacements
                 p("Before custom output replacements: %s" % (episode.generateFilename(preview_orig_filename = False)))
-    
+
             p("New filename: %s" % newName)
-    
+
             if Config['always_rename']:
                 doRenameFile(cnamer, newName)
                 if Config['move_files_enable']:
@@ -217,9 +219,9 @@ def processFile(tvdb_instance, episode):
                     else:
                         doMoveFile(cnamer = cnamer, destDir = getMoveDestination(episode))
                 return
-    
+
             ans = confirm("Rename?", options = ['y', 'n', 'a', 'q'], default = 'y')
-    
+
             if ans == "a":
                 p("Always renaming")
                 Config['always_rename'] = True
@@ -234,7 +236,7 @@ def processFile(tvdb_instance, episode):
                 p("Skipping")
             else:
                 p("Invalid input, skipping")
-    
+
             if shouldRename:
                 doRenameFile(cnamer, newName)
 
@@ -300,8 +302,9 @@ def tvnamer(paths):
         except InvalidFilename, e:
             warn("Invalid filename: %s" % e)
         else:
-            if episode.seriesname is None and Config['force_name'] is None:
-                warn("Parsed filename did not contain series name (and --name not specified), skipping: %s" % cfile)
+            if episode.seriesname is None and Config['force_name'] is None and Config['series_id'] is None:
+                warn("Parsed filename did not contain series name (and --name or --series-id not specified), skipping: %s" % cfile)
+
             else:
                 episodes_found.append(episode)
 
@@ -357,7 +360,7 @@ def main():
     if configToLoad is not None:
         p("Loading config: %s" % (configToLoad))
         try:
-            loadedConfig = json.load(open(configToLoad))
+            loadedConfig = json.load(open(os.path.expanduser(configToLoad)))
         except ValueError, e:
             p("Error loading config: %s" % e)
             opter.exit(1)
@@ -380,7 +383,7 @@ def main():
         del configToSave['showconfig']
         json.dump(
             configToSave,
-            open(opts.saveconfig, "w+"),
+            open(os.path.expanduser(opts.saveconfig), "w+"),
             sort_keys=True,
             indent=4)
 
@@ -388,8 +391,7 @@ def main():
 
     # Show config argument
     if opts.showconfig:
-        for k, v in opts.__dict__.items():
-            p(k, "=", str(v))
+        print json.dumps(opts.__dict__, sort_keys=True, indent=2)
         return
 
     # Process values
@@ -406,6 +408,9 @@ def main():
         p("#" * 20)
         opter.exit(0)
 
+    if Config['titlecase_filename'] and Config['lowercase_filename']:
+        warnings.warn("Setting 'lowercase_filename' clobbers 'titlecase_filename' option")
+
     if len(args) == 0:
         opter.error("No filenames or directories supplied")
 
