@@ -10,6 +10,7 @@ import sys
 import shutil
 import logging
 import platform
+import errno
 
 from tvdb_api import (tvdb_error, tvdb_shownotfound, tvdb_seasonnotfound,
 tvdb_episodenotfound, tvdb_attributenotfound, tvdb_userabort)
@@ -34,6 +35,12 @@ def warn(text):
     p(text, file = sys.stderr)
 
 
+def split_extension(filename):
+    base = re.sub(Config["extension_pattern"], "", filename)
+    ext = filename.replace(base, "")
+    return base, ext
+
+
 def _applyReplacements(cfile, replacements):
     """Applies custom replacements.
 
@@ -43,11 +50,21 @@ def _applyReplacements(cfile, replacements):
     "replacement", and (optional) "is_regex"
     """
     for rep in replacements:
+        if not rep.get('with_extension', False):
+            # By default, preserve extension
+            cfile, cext = split_extension(cfile)
+        else:
+            cfile = cfile
+            cext = ""
+
         if 'is_regex' in rep and rep['is_regex']:
             cfile = re.sub(rep['match'], rep['replacement'], cfile)
         else:
             cfile = cfile.replace(rep['match'], rep['replacement'])
 
+        # Rejoin extension (cext might be empty-string)
+        cfile = cfile + cext
+
     return cfile
 
 
@@ -81,6 +98,7 @@ def cleanRegexedSeriesName(seriesname):
     >>> cleanRegexedSeriesName("an_example_1.0_test")
     'an example 1.0 test'
     """
+    # TODO: Could this be made to clean "Hawaii.Five-0.2010" into "Hawaii Five-0 2010"?
     seriesname = re.sub("(\D)[.](\D)", "\\1 \\2", seriesname)
     seriesname = re.sub("(\D)[.]", "\\1 ", seriesname)
     seriesname = re.sub("[.](\D)", " \\1", seriesname)
@@ -91,10 +109,10 @@ def cleanRegexedSeriesName(seriesname):
 
 def replaceInputSeriesName(seriesname):
     """allow specified replacements of series names
-    
+
     in cases where default filenames match the wrong series,
     e.g. missing year gives wrong answer, or vice versa
-    
+
     This helps the TVDB query get the right match.
     """
     for pat, replacement in Config['input_series_replacements'].iteritems():
@@ -102,14 +120,15 @@ def replaceInputSeriesName(seriesname):
             return replacement
     return seriesname
 
+
 def replaceOutputSeriesName(seriesname):
     """transform TVDB series names
-    
+
     after matching from TVDB, transform the series name for desired abbreviation, etc.
-    
+
     This affects the output filename.
     """
-    
+
     return Config['output_series_replacements'].get(seriesname, seriesname)
 
 
@@ -133,6 +152,7 @@ def handleYear(year):
     else:
         return 1900 + year
 
+
 class FileFinder(object):
     """Given a file, it will verify it exists. Given a folder it will descend
     one level into it and return a list of files, unless the recursive argument
@@ -180,6 +200,7 @@ class FileFinder(object):
         if len(self.with_extension) == 0:
             return True
 
+        # don't use split_extension here (otherwise valid_extensions is useless)!
         _, extension = os.path.splitext(fname)
         for cext in self.with_extension:
             cext = ".%s" % cext
@@ -192,8 +213,12 @@ class FileFinder(object):
         """Checks if the filename (optionally excluding extension)
         matches filename_blacklist
 
-        self.with_blacklist should be a list of dicts, where each dict
-        contains:
+        self.with_blacklist should be a list of strings and/or dicts:
+
+        a string, specifying an exact filename to ignore
+        "filename_blacklist": [".DS_Store", "Thumbs.db"],
+
+        a dictionary, where each dict contains:
 
         Key 'match' - (if the filename matches the pattern, the filename
         is blacklisted)
@@ -213,9 +238,15 @@ class FileFinder(object):
             return False
 
         fdir, fullname = os.path.split(filepath)
-        fname, fext = os.path.splitext(fullname)
+        fname, fext = split_extension(fullname)
 
         for fblacklist in self.with_blacklist:
+            if isinstance(fblacklist, basestring):
+                if fullname == fblacklist:
+                    return True
+                else:
+                    continue
+
             if "full_path" in fblacklist and fblacklist["full_path"]:
                 to_check = filepath
             else:
@@ -229,7 +260,7 @@ class FileFinder(object):
                 if m is not None:
                     return True
             else:
-                m =  fblacklist["match"] in to_check
+                m = fblacklist["match"] in to_check
                 if m:
                     return True
         else:
@@ -311,10 +342,15 @@ class FileParser(object):
                     # Multiple episodes, regex specifies start and end number
                     start = int(match.group('episodenumberstart'))
                     end = int(match.group('episodenumberend'))
-                    if start > end:
+                    if end - start > 5:
+                        warn("WARNING: %s episodes detected in file: %s, confused by numeric episode name, using first match: %s" %(end - start, filename, start))
+                        episodenumbers = [start]
+                    elif start > end:
                         # Swap start and end
                         start, end = end, start
-                    episodenumbers = range(start, end + 1)
+                        episodenumbers = range(start, end + 1)
+                    else:
+                        episodenumbers = range(start, end + 1)
 
                 elif 'episodenumber' in namedgroups:
                     episodenumbers = [int(match.group('episodenumber')), ]
@@ -387,10 +423,14 @@ class FileParser(object):
             raise InvalidFilename(emsg)
 
 
-def formatEpisodeName(names, join_with):
-    """Takes a list of episode names, formats them into a string.
+def formatEpisodeName(names, join_with, multiep_format):
+    """
+    Takes a list of episode names, formats them into a string.
+
     If two names are supplied, such as "Pilot (1)" and "Pilot (2)", the
-    returned string will be "Pilot (1-2)"
+    returned string will be "Pilot (1-2)". Note that the first number
+    is not required, for example passing "Pilot" and "Pilot (2)" will
+    also result in returning "Pilot (1-2)".
 
     If two different episode names are found, such as "The first", and
     "Something else" it will return "The first, Something else"
@@ -398,24 +438,23 @@ def formatEpisodeName(names, join_with):
     if len(names) == 1:
         return names[0]
 
-    found_names = []
+    found_name = ""
     numbers = []
     for cname in names:
-        number = re.match("(.*) \(([0-9]+)\)$", cname)
-        if number:
-            epname, epno = number.group(1), number.group(2)
-            if len(found_names) > 0 and epname not in found_names:
-                return join_with.join(names)
-            found_names.append(epname)
-            numbers.append(int(epno))
-        else:
+        match = re.match("(.*) \(([0-9]+)\)$", cname)
+        if found_name != "" and (not match or epname != found_name):
             # An episode didn't match
             return join_with.join(names)
 
-    names = []
-    start, end = min(numbers), max(numbers)
-    names.append("%s (%d-%d)" % (found_names[0], start, end))
-    return join_with.join(names)
+        if match:
+            epname, epno = match.group(1), match.group(2)
+        else: # assume that this is the first episode, without number
+            epname = cname
+            epno = 1
+        found_name = epname
+        numbers.append(int(epno))
+
+    return multiep_format % {'epname': found_name, 'episodemin': min(numbers), 'episodemax': max(numbers)}
 
 
 def makeValidFilename(value, normalize_unicode = False, windows_safe = False, custom_blacklist = None, replace_with = "_"):
@@ -452,7 +491,7 @@ def makeValidFilename(value, normalize_unicode = False, windows_safe = False, cu
         value = "_" + value
 
     # Treat extension seperatly
-    value, extension = os.path.splitext(value)
+    value, extension = split_extension(value)
 
     # Remove any null bytes
     value = value.replace("\0", "")
@@ -535,7 +574,6 @@ class EpisodeInfo(object):
     CFG_KEY_WITH_EP = "filename_with_episode"
     CFG_KEY_WITHOUT_EP = "filename_without_episode"
 
-
     def __init__(self,
         seriesname,
         seasonnumber,
@@ -568,14 +606,13 @@ class EpisodeInfo(object):
             self.filename, self.extension = None, None
         else:
             self.filepath, self.filename = os.path.split(value)
-            self.filename, self.extension = os.path.splitext(self.filename)
-            self.extension = self.extension.replace(".", "")
+            self.filename, self.extension = split_extension(self.filename)
 
     fullpath = property(fullpath_get, fullpath_set)
 
     @property
     def fullfilename(self):
-        return u"%s.%s" % (self.filename, self.extension)
+        return u"%s%s" % (self.filename, self.extension)
 
     def sortable_info(self):
         """Returns a tuple of sortable information
@@ -660,7 +697,7 @@ class EpisodeInfo(object):
                         if int(e['absolute_number']) == cepno:
                             epnames.append(e['episodename'])
                             unsure = False
-                    # If unsure error out            
+                    # If unsure error out
                     if unsure:
                         raise EpisodeNotFound(
                             "No episode actually matches %s, found %s results instead" % (cepno, len(sr)))
@@ -681,7 +718,6 @@ class EpisodeInfo(object):
 
         self.episodename = epnames
 
-
     def getepdata(self):
         """
         Uses the following config options:
@@ -697,7 +733,7 @@ class EpisodeInfo(object):
         if self.extension is None:
             prep_extension = ''
         else:
-            prep_extension = '.%s' % self.extension
+            prep_extension = self.extension
 
         epdata = {
             'seriesname': self.seriesname,
@@ -723,10 +759,14 @@ class EpisodeInfo(object):
             if isinstance(self.episodename, list):
                 epdata['episodename'] = formatEpisodeName(
                     self.episodename,
-                    join_with = Config['multiep_join_name_with']
-                )
+                    join_with = Config['multiep_join_name_with'],
+                    multiep_format = Config['multiep_format'])
             fname = Config[self.CFG_KEY_WITH_EP] % epdata
 
+        if Config['titlecase_filename']:
+            from _titlecase import titlecase
+            fname = titlecase(fname)
+
         if lowercase or Config['lowercase_filename']:
             fname = fname.lower()
 
@@ -735,10 +775,7 @@ class EpisodeInfo(object):
             return fname
 
         if len(Config['output_filename_replacements']) > 0:
-            # Only apply replacements to filename, not extension
-            splitname, splitext = os.path.splitext(fname)
-            newname = applyCustomOutputReplacements(splitname)
-            fname = newname + splitext
+            fname = applyCustomOutputReplacements(fname)
 
         return makeValidFilename(
             fname,
@@ -768,7 +805,7 @@ class DatedEpisodeInfo(EpisodeInfo):
         self.episodenumbers = episodenumbers
         self.episodename = episodename
         self.fullpath = filename
-        
+
         if filename is not None:
             # Remains untouched, for use when renaming file
             self.originalfilename = os.path.basename(filename)
@@ -802,8 +839,8 @@ class DatedEpisodeInfo(EpisodeInfo):
         if isinstance(self.episodename, list):
             prep_episodename = formatEpisodeName(
                 self.episodename,
-                join_with = Config['multiep_join_name_with']
-            )
+                join_with = Config['multiep_join_name_with'],
+                multiep_format = Config['multiep_format'])
         else:
             prep_episodename = self.episodename
 
@@ -811,7 +848,7 @@ class DatedEpisodeInfo(EpisodeInfo):
         if self.extension is None:
             prep_extension = ''
         else:
-            prep_extension = '.%s' % self.extension
+            prep_extension = self.extension
 
         epdata = {
             'seriesname': self.seriesname,
@@ -866,7 +903,7 @@ class NoSeasonEpisodeInfo(EpisodeInfo):
         if self.extension is None:
             prep_extension = ''
         else:
-            prep_extension = '.%s' % self.extension
+            prep_extension = self.extension
 
         epdata = {
             'seriesname': self.seriesname,
@@ -884,7 +921,6 @@ class AnimeEpisodeInfo(NoSeasonEpisodeInfo):
     CFG_KEY_WITH_EP_NO_CRC = "filename_anime_with_episode_without_crc"
     CFG_KEY_WITHOUT_EP_NO_CRC = "filename_anime_without_episode_without_crc"
 
-
     def generateFilename(self, lowercase = False, preview_orig_filename = False):
         epdata = self.getepdata()
 
@@ -911,8 +947,8 @@ class AnimeEpisodeInfo(NoSeasonEpisodeInfo):
             if isinstance(self.episodename, list):
                 epdata['episodename'] = formatEpisodeName(
                     self.episodename,
-                    join_with = Config['multiep_join_name_with']
-                )
+                    join_with = Config['multiep_join_name_with'],
+                    multiep_format = Config['multiep_format'])
 
         fname = Config[cfgkey] % epdata
 
@@ -925,10 +961,7 @@ class AnimeEpisodeInfo(NoSeasonEpisodeInfo):
             return fname
 
         if len(Config['output_filename_replacements']) > 0:
-            # Only apply replacements to filename, not extension
-            splitname, splitext = os.path.splitext(fname)
-            newname = applyCustomOutputReplacements(splitname)
-            fname = newname + splitext
+            fname = applyCustomOutputReplacements(fname)
 
         return makeValidFilename(
             fname,
@@ -945,7 +978,47 @@ def same_partition(f1, f2):
 
 
 def delete_file(fpath):
-    raise NotImplementedError("delete_file not yet implimented")
+    """On OS X: Trashes a path using the Finder, via OS X's Scripting Bridge.
+
+    On other platforms: unlinks file.
+    """
+
+    try:
+        from AppKit import NSURL
+        from ScriptingBridge import SBApplication
+    except ImportError:
+        log().debug("Deleting %r" % fpath)
+        os.unlink(fpath)
+    else:
+        log().debug("Trashing %r" % fpath)
+        targetfile = NSURL.fileURLWithPath_(fpath)
+        finder = SBApplication.applicationWithBundleIdentifier_("com.apple.Finder")
+        items = finder.items().objectAtLocation_(targetfile)
+        items.delete()
+
+
+def rename_file(old, new):
+    p("rename %s to %s" % (old, new))
+    stat = os.stat(old)
+    os.rename(old, new)
+    try:
+        os.utime(new, (stat.st_atime, stat.st_mtime))
+    except OSError, ex:
+        if ex.errno == errno.EPERM:
+            warn("WARNING: Could not preserve times for %s "
+                 "(owner UID mismatch?)" % new)
+        else:
+            raise
+
+def copy_file(old, new):
+    p("copy %s to %s" % (old, new))
+    shutil.copyfile(old, new)
+    shutil.copystat(old, new)
+
+
+def symlink_file(target, name):
+    p("symlink %s to %s" % (name, target))
+    os.symlink(target, name)
 
 
 class Renamer(object):
@@ -955,29 +1028,15 @@ class Renamer(object):
     def __init__(self, filename):
         self.filename = os.path.abspath(filename)
 
-    def newName(self, newName, force = False):
-        """Renames a file, keeping the path the same.
-        """
-        filepath, filename = os.path.split(self.filename)
-        filename, _ = os.path.splitext(filename)
-
-        newpath = os.path.join(filepath, newName)
-
-        if os.path.isfile(newpath):
-            # If the destination exists, raise exception unless force is True
-            if not force:
-                raise OSError("File %s already exists, not forcefully renaming %s" % (
-                    newpath, self.filename))
-
-        os.rename(self.filename, newpath)
-        self.filename = newpath
-
-    def newPath(self, new_path = None, new_fullpath = None, force = False, always_copy = False, always_move = False, create_dirs = True, getPathPreview = False):
+    def newPath(self, new_path = None, new_fullpath = None, force = False, always_copy = False, always_move = False, leave_symlink = False, create_dirs = True, getPathPreview = False):
         """Moves the file to a new path.
 
         If it is on the same partition, it will be moved (unless always_copy is True)
-        If it is on a different partition, it will be copied.
+        If it is on a different partition, it will be copied, and the original
+        only deleted if always_move is True.
         If the target file already exists, it will raise OSError unless force is True.
+        If it was moved, a symlink will be left behind with the original name
+        pointing to the file's new destination if leave_symlink is True.
         """
 
         if always_copy and always_move:
@@ -986,9 +1045,8 @@ class Renamer(object):
         if (new_path is None and new_fullpath is None) or (new_path is not None and new_fullpath is not None):
             raise ValueError("Specify only new_dir or new_fullpath")
 
+        old_dir, old_filename = os.path.split(self.filename)
         if new_path is not None:
-            old_dir, old_filename = os.path.split(self.filename)
-
             # Join new filepath to old one (to handle realtive dirs)
             new_dir = os.path.abspath(os.path.join(old_dir, new_path))
 
@@ -996,8 +1054,6 @@ class Renamer(object):
             new_fullpath = os.path.join(new_dir, old_filename)
 
         else:
-            old_dir, old_filename = os.path.split(self.filename)
-
             # Join new filepath to old one (to handle realtive dirs)
             new_fullpath = os.path.abspath(os.path.join(old_dir, new_fullpath))
 
@@ -1015,12 +1071,14 @@ class Renamer(object):
             return new_fullpath
 
         if create_dirs:
-            p("Creating directory %s" % new_dir)
             try:
                 os.makedirs(new_dir)
             except OSError, e:
                 if e.errno != 17:
                     raise
+            else:
+                p("Created directory %s" % new_dir)
+
 
         if os.path.isfile(new_fullpath):
             # If the destination exists, raise exception unless force is True
@@ -1031,19 +1089,24 @@ class Renamer(object):
         if same_partition(self.filename, new_dir):
             if always_copy:
                 # Same partition, but forced to copy
-                p("copy %s to %s" % (self.filename, new_fullpath))
-                shutil.copyfile(self.filename, new_fullpath)
+                copy_file(self.filename, new_fullpath)
             else:
                 # Same partition, just rename the file to move it
-                p("move %s to %s" % (self.filename, new_fullpath))
-                os.rename(self.filename, new_fullpath)
+                rename_file(self.filename, new_fullpath)
+
+                # Leave a symlink behind if configured to do so
+                if leave_symlink:
+                    symlink_file(new_fullpath, self.filename)
         else:
             # File is on different partition (different disc), copy it
-            p("copy %s to %s" % (self.filename, new_fullpath))
-            shutil.copyfile(self.filename, new_fullpath)
+            copy_file(self.filename, new_fullpath)
             if always_move:
                 # Forced to move file, we just trash old file
                 p("Deleting %s" % (self.filename))
                 delete_file(self.filename)
 
+                # Leave a symlink behind if configured to do so
+                if leave_symlink:
+                    symlink_file(new_fullpath, self.filename)
+
         self.filename = new_fullpath
